@@ -140,6 +140,12 @@ int (*xstat) ();
 /* Status value to return to system. */
 int exit_status;
 
+/* If true, we ignore the problem where we find that a directory entry 
+ * no longer exists by the time we get around to processing it.
+ */
+boolean ignore_readdir_race;
+
+
 #ifdef DEBUG_STAT
 static int
 debug_stat (file, bufp)
@@ -176,6 +182,7 @@ main (int argc, char **argv)
   full_days = false;
   no_leaf_check = false;
   stay_on_filesystem = false;
+  ignore_readdir_race = false;
   exit_status = 0;
   dereference = false;
 #ifdef DEBUG_STAT
@@ -184,7 +191,20 @@ main (int argc, char **argv)
   xstat = lstat;
 #endif /* !DEBUG_STAT */
 
+#if 0  
   human_block_size (getenv ("FIND_BLOCK_SIZE"), 0, &output_block_size);
+#else
+  if (getenv("POSIXLY_CORRECT"))
+    output_block_size = 512;
+  else
+    output_block_size = 1024;
+  
+  if (getenv("FIND_BLOCK_SIZE"))
+    {
+      error (1, errno, _("The environment variable FIND_BLOCK_SIZE is not supported, the only thing that affects the block size is the POSIXLY_CORRECT environment variable"));
+    }
+  
+#endif
 
 #ifdef DEBUG
   printf ("cur_day_start = %s", ctime (&cur_day_start));
@@ -250,6 +270,15 @@ main (int argc, char **argv)
   /* Done parsing the predicates.  Build the evaluation tree. */
   cur_pred = predicates;
   eval_tree = get_expr (&cur_pred, NO_PREC);
+
+  /* Check if we have any left-over predicates (this fixes
+   * Debian bug #185202).
+   */
+  if (cur_pred != NULL)
+    {
+      error (1, 0, _("unexpected extra predicate"));
+    }
+  
 #ifdef	DEBUG
   printf (_("Eval Tree:\n"));
   print_tree (eval_tree, 0);
@@ -331,8 +360,11 @@ process_top_path (char *pathname)
     {
       if (chdir (pathname) < 0)
 	{
-	  error (0, errno, "%s", pathname);
-	  exit_status = 1;
+	  if (!ignore_readdir_race || (errno != ENOENT) )
+	    {
+	      error (0, errno, "%s", pathname);
+	      exit_status = 1;
+	    }
 	  return;
 	}
 
@@ -400,8 +432,11 @@ process_path (char *pathname, char *name, boolean leaf, char *parent)
     {
       if ((*xstat) (name, &stat_buf) != 0)
 	{
-	  error (0, errno, "%s", pathname);
-	  exit_status = 1;
+	  if (!ignore_readdir_race || (errno != ENOENT) )
+	    {
+	      error (0, errno, "%s", pathname);
+	      exit_status = 1;
+	    }
 	  return 0;
 	}
       have_stat = true;
