@@ -82,6 +82,7 @@ static boolean parse_iname PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_inum PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_ipath PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_iregex PARAMS((char *argv[], int *arg_ptr));
+static boolean parse_iwholename PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_links PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_lname PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_ls PARAMS((char *argv[], int *arg_ptr));
@@ -113,7 +114,10 @@ static boolean parse_uid PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_used PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_user PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_version PARAMS((char *argv[], int *arg_ptr));
+static boolean parse_wholename PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_xdev PARAMS((char *argv[], int *arg_ptr));
+static boolean parse_ignore_race PARAMS((char *argv[], int *arg_ptr));
+static boolean parse_noignore_race PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_xtype PARAMS((char *argv[], int *arg_ptr));
 
 static boolean insert_regex PARAMS((char *argv[], int *arg_ptr, boolean ignore_case));
@@ -176,11 +180,13 @@ static struct parser_table const parse_table[] =
   {"group", parse_group},
   {"help", parse_help},		/* GNU */
   {"-help", parse_help},	/* GNU */
+  {"ignore_readdir_race", parse_ignore_race},	/* GNU */
   {"ilname", parse_ilname},	/* GNU */
   {"iname", parse_iname},	/* GNU */
   {"inum", parse_inum},		/* GNU, Unix */
-  {"ipath", parse_ipath},	/* GNU */
+  {"ipath", parse_ipath},	/* GNU, deprecated in favour of iwholename */
   {"iregex", parse_iregex},	/* GNU */
+  {"iwholename", parse_iwholename}, /* GNU */
   {"links", parse_links},
   {"lname", parse_lname},	/* GNU */
   {"ls", parse_ls},		/* GNU, Unix */
@@ -197,10 +203,11 @@ static struct parser_table const parse_table[] =
   {"noleaf", parse_noleaf},	/* GNU */
   {"nogroup", parse_nogroup},
   {"nouser", parse_nouser},
+  {"noignore_readdir_race", parse_noignore_race},	/* GNU */
   {"o", parse_or},
   {"or", parse_or},		/* GNU */
   {"ok", parse_ok},
-  {"path", parse_path},		/* GNU, HP-UX */
+  {"path", parse_path},		/* GNU, HP-UX, GNU prefers wholename */
   {"perm", parse_perm},
   {"print", parse_print},
   {"print0", parse_print0},	/* GNU */
@@ -215,6 +222,8 @@ static struct parser_table const parse_table[] =
   {"user", parse_user},
   {"version", parse_version},	/* GNU */
   {"-version", parse_version},	/* GNU */
+  {"wholename", parse_wholename}, /* GNU, replaces -path */
+
   {"xdev", parse_xdev},
   {"xtype", parse_xtype},	/* GNU */
   {0, 0}
@@ -563,19 +572,21 @@ operators (decreasing precedence; -and is implicit where no others are given):\n
       EXPR1 -o EXPR2 EXPR1 -or EXPR2 EXPR1 , EXPR2\n\
 options (always true): -daystart -depth -follow --help\n\
       -maxdepth LEVELS -mindepth LEVELS -mount -noleaf --version -xdev\n\
-tests (N can be +N or -N or N): -amin N -anewer FILE -atime N -cmin N\n"));
+tests (N can be +N or -N or N): -amin N -anewer FILE -atime N -cmin N"));
   puts (_("\
       -cnewer FILE -ctime N -empty -false -fstype TYPE -gid N -group NAME\n\
-      -ilname PATTERN -iname PATTERN -inum N -ipath PATTERN -iregex PATTERN\n\
-      -links N -lname PATTERN -mmin N -mtime N -name PATTERN -newer FILE\n"));
+      -ilname PATTERN -iname PATTERN -inum N -iwholename PATTERN -iregex PATTERN\n\
+      -links N -lname PATTERN -mmin N -mtime N -name PATTERN -newer FILE"));
   puts (_("\
       -nouser -nogroup -path PATTERN -perm [+-]MODE -regex PATTERN\n\
-      -size N[bckw] -true -type [bcdpfls] -uid N -used N -user NAME\n\
-      -xtype [bcdpfls]\n"));
+      -wholename PATTERN -size N[bcwkMG] -true -type [bcdpflsD] -uid N\n\
+      -used N -user NAME -xtype [bcdpfls]\n"));
   puts (_("\
 actions: -exec COMMAND ; -fprint FILE -fprint0 FILE -fprintf FILE FORMAT\n\
       -ok COMMAND ; -print -print0 -printf FORMAT -prune -ls\n"));
-  puts (_("\nReport bugs to <bug-findutils@gnu.org>."));
+  puts (_("Report (and track progress on fixing) bugs via the findutils bug-reporting\n\
+page at http://savannah.gnu.org/ or, if you have no web access, by sending\n\
+email to <bug-findutils@gnu.org>."));
   exit (0);
 }
 
@@ -612,8 +623,22 @@ parse_inum (char **argv, int *arg_ptr)
   return (insert_num (argv, arg_ptr, pred_inum));
 }
 
+/* -ipath is deprecated (at RMS's request) in favour of 
+ * -iwholename.   See the node "GNU Manuals" in standards.texi
+ * for the rationale for this (basically, GNU prefers the use 
+ * of the phrase "file name" to "path name"
+ */
 static boolean
 parse_ipath (char **argv, int *arg_ptr)
+{
+  error (0, 0,
+	 _("warning: the predicate -ipath is deprecated; please use -iwholename instead."));
+  
+  return parse_iwholename(argv, arg_ptr);
+}
+
+static boolean
+parse_iwholename (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
@@ -896,8 +921,22 @@ parse_or (char **argv, int *arg_ptr)
   return (true);
 }
 
+/* -path is deprecated (at RMS's request) in favour of 
+ * -iwholename.   See the node "GNU Manuals" in standards.texi
+ * for the rationale for this (basically, GNU prefers the use 
+ * of the phrase "file name" to "path name".
+ *
+ * We do not issue a warning that this usage is deprecated
+ * since HPUX find supports this predicate also.
+ */
 static boolean
 parse_path (char **argv, int *arg_ptr)
+{
+  return parse_wholename(argv, arg_ptr);
+}
+
+static boolean
+parse_wholename (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
@@ -1085,6 +1124,16 @@ parse_size (char **argv, int *arg_ptr)
       argv[*arg_ptr][len - 1] = '\0';
       break;
 
+    case 'M':			/* Megabytes */
+      blksize = 1024*1024;
+      argv[*arg_ptr][len - 1] = '\0';
+      break;
+
+    case 'G':			/* Gigabytes */
+      blksize = 1024*1024*1024;
+      argv[*arg_ptr][len - 1] = '\0';
+      break;
+
     case 'w':
       blksize = 2;
       argv[*arg_ptr][len - 1] = '\0';
@@ -1202,6 +1251,20 @@ parse_xdev (char **argv, int *arg_ptr)
   return true;
 }
 
+static boolean
+parse_ignore_race (char **argv, int *arg_ptr)
+{
+  ignore_readdir_race = true;
+  return true;
+}
+
+static boolean
+parse_noignore_race (char **argv, int *arg_ptr)
+{
+  ignore_readdir_race = false;
+  return true;
+}
+
 static boolean
 parse_xtype (char **argv, int *arg_ptr)
 {
@@ -1579,8 +1642,22 @@ insert_time (char **argv, int *arg_ptr, PFB pred)
     return (false);
   if (!get_num_days (argv[*arg_ptr], &num_days, &c_type))
     return (false);
-  t = (cur_day_start - num_days * DAYSECS
-       + ((c_type == COMP_GT) ? DAYSECS - 1 : 0));
+
+  /* Figure out the timestamp value we are looking for. */
+  t = ( cur_day_start - num_days * DAYSECS
+		   + ((c_type == COMP_GT) ? DAYSECS - 1 : 0));
+#if 1
+  intmax_t val = ( (intmax_t)cur_day_start - num_days * DAYSECS
+		   + ((c_type == COMP_GT) ? DAYSECS - 1 : 0));
+  t = val;
+  
+  /* Check for possibility of an overflow */
+  if ( (intmax_t)t != val ) 
+    {
+      error (1, 0, "arithmetic overflow while converting %s days to a number of seconds", argv[*arg_ptr]);
+    }
+#endif
+  
   our_pred = insert_primary (pred);
   our_pred->args.info.kind = c_type;
   our_pred->args.info.negative = t < 0;
