@@ -560,7 +560,8 @@ pred_fprintf (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 	case 'b':		/* size in 512-byte blocks */
 	  fprintf (fp, segment->text,
 		   human_readable ((uintmax_t) ST_NBLOCKS (*stat_buf),
-				   hbuf, ST_NBLOCKSIZE, 512));
+				   hbuf, human_ceiling,
+				   ST_NBLOCKSIZE, 512));
 	  break;
 	case 'c':		/* ctime in `ctime' format */
 	  fprintf (fp, segment->text, ctime_format (stat_buf->st_ctime));
@@ -590,7 +591,8 @@ pred_fprintf (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 	  }
 	case 'G':		/* GID number */
 	  fprintf (fp, segment->text,
-		   human_readable ((uintmax_t) stat_buf->st_gid, hbuf, 1, 1));
+		   human_readable ((uintmax_t) stat_buf->st_gid, hbuf,
+				   human_ceiling, 1, 1));
 	  break;
 	case 'h':		/* leading directories part of path */
 	  {
@@ -616,12 +618,15 @@ pred_fprintf (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 	  }
 	case 'i':		/* inode number */
 	  fprintf (fp, segment->text,
-		   human_readable ((uintmax_t) stat_buf->st_ino, hbuf, 1, 1));
+		   human_readable ((uintmax_t) stat_buf->st_ino, hbuf,
+				   human_ceiling,
+				   1, 1));
 	  break;
 	case 'k':		/* size in 1K blocks */
 	  fprintf (fp, segment->text,
 		   human_readable ((uintmax_t) ST_NBLOCKS (*stat_buf),
-				   hbuf, ST_NBLOCKSIZE, 1024));
+				   hbuf, human_ceiling,
+				   ST_NBLOCKSIZE, 1024)); 
 	  break;
 	case 'l':		/* object of symlink */
 #ifdef S_ISLNK
@@ -676,7 +681,9 @@ pred_fprintf (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 	case 'n':		/* number of links */
 	  fprintf (fp, segment->text,
 		   human_readable ((uintmax_t) stat_buf->st_nlink,
-				   hbuf, 1, 1));
+				   hbuf,
+				   human_ceiling,
+				   1, 1));
 	  break;
 	case 'p':		/* pathname */
 	  fprintf (fp, segment->text, pathname);
@@ -699,7 +706,7 @@ pred_fprintf (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 	case 's':		/* size in bytes */
 	  fprintf (fp, segment->text,
 		   human_readable ((uintmax_t) stat_buf->st_size,
-				   hbuf, 1, 1));
+				   hbuf, human_ceiling, 1, 1));
 	  break;
 	case 't':		/* mtime in `ctime' format */
 	  fprintf (fp, segment->text, ctime_format (stat_buf->st_mtime));
@@ -719,7 +726,8 @@ pred_fprintf (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 	  }
 	case 'U':		/* UID number */
 	  fprintf (fp, segment->text,
-		   human_readable ((uintmax_t) stat_buf->st_uid, hbuf, 1, 1));
+		   human_readable ((uintmax_t) stat_buf->st_uid, hbuf,
+				   human_ceiling, 1, 1));
 	  break;
 	}
     }
@@ -1319,7 +1327,8 @@ format_date (time_t when, int kind)
   else
     {
       uintmax_t w = when;
-      char *p = human_readable (when < 0 ? -w : w, buf + 1, 1, 1);
+      char *p = human_readable (when < 0 ? -w : w, buf + 1,
+				human_ceiling, 1, 1);
       if (when < 0)
 	*--p = '-';
       return p;
