@@ -59,6 +59,7 @@
 #undef PACKAGE
 #include <config.h>
 #include <stdio.h>
+#include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <time.h>
@@ -136,7 +137,7 @@ void error ();
 
 /* Read in a 16-bit int, high byte first (network byte order).  */
 
-static int
+static short
 get_short (fp)
      FILE *fp;
 {
@@ -367,12 +368,21 @@ locate (pathpart, dbfile, ignore_case)
       else
 	{
 	  if (c == LOCATEDB_ESCAPE)
-	    count += get_short (fp);
+	    count += (short)get_short (fp);
 	  else if (c > 127)
 	    count += c - 256;
 	  else
 	    count += c;
 
+	  if (count > strlen(path))
+	    {
+	      /* This should not happen generally , but since we're
+	       * reading in data which is outside our control, we
+	       * cannot prevent it.
+	       */
+	      error(1, 0, _("locate database `%s' is corrupt or invalid"), dbfile);
+	    }
+	  
 	  /* Overlay the old path with the remainder of the new.  */
 	  nread = locate_read_str (&path, &pathsize, fp, 0, count); 
 	  if (nread < 0)
@@ -392,66 +402,47 @@ locate (pathpart, dbfile, ignore_case)
       /* Search backward starting at the end of the path we just read in,
 	 for the character at the end of the last glob-free subpattern
 	 in PATHPART.  */
-      if (ignore_case)
-	{
-          for (prev_fast_match = false; s >= cutoff; s--)
-	    /* Fast first char check. */
-	    if (TOLOWER(*s) == *patend)
-	      {
-	        char *s2;		/* Scan the path we read in. */
-	        register char *p2;	/* Scan `patend'.  */
+      for(prev_fast_match=false; s>=cutoff; s--)
+        {
+          char *s2;   /* Scan the path we read in. */
+          register char *p2;  /* Scan `patend'.  */
 
-	        for (s2 = s - 1, p2 = patend - 1; *p2 != '\0' && TOLOWER(*s2) == *p2;
-		     s2--, p2--)
-	          ;
-		if (*p2 == '\0')
-		  {
-		    /* Success on the fast match.  Compare the whole pattern
-		       if it contains globbing characters.  */
-		    prev_fast_match = true;
-		    if (globflag == false || fnmatch (pathpart, path, FNM_CASEFOLD) == 0)
-		      {
-			if (!check_existence || stat(path, &st) == 0)
-			  {
-			    puts (path);
-			    ++printed;
-			  }
-		      }
-		    break;
-		  }
-	      }
-	}
-      else {
-	
-      for (prev_fast_match = false; s >= cutoff; s--)
-	/* Fast first char check. */
-	if (*s == *patend)
-	  {
-	    char *s2;		/* Scan the path we read in. */
-	    register char *p2;	/* Scan `patend'.  */
+          /* Fast first char check. */
+          if(ignore_case)
+            {
+              if(TOLOWER(*s)!=*patend)
+                continue;
+            }
+          else if(*s!=*patend)
+            continue;
 
-	    for (s2 = s - 1, p2 = patend - 1; *p2 != '\0' && *s2 == *p2;
-					       s2--, p2--)
-	      ;
-	    if (*p2 == '\0')
-	      {
-		/* Success on the fast match.  Compare the whole pattern
-		   if it contains globbing characters.  */
-		prev_fast_match = true;
-		if (globflag == false || fnmatch (pathpart, path,
-						  0) == 0)
-		  {
-		    if (!check_existence || stat(path, &st) == 0)
-		      {
-			puts (path);
-			++printed;
-		      }
-		  }
-		break;
-	      }
-	  }
-      }
-      
+          if(ignore_case)
+            for(s2=s-1, p2=patend-1; *p2!='\0' && TOLOWER(*s2)==*p2; s2--, p2--);
+          else
+            for(s2=s-1, p2=patend-1; *p2!='\0' && *s2==*p2; s2--, p2--);
+
+          if(*p2!='\0')
+            continue;
+          /*  Success on the fast match.  Compare the whole pattern
+              if it contains globbing characters.  */
+          prev_fast_match = true;
+          if(globflag)
+            {
+              if(ignore_case)
+                {
+                  if(fnmatch(pathpart,basename(path),FNM_CASEFOLD)!=0)
+                    break;
+                }
+              else
+                if(fnmatch(pathpart,basename(path),0)!=0)
+                  break;
+            }
+          if(check_existence && stat(path,&st)!=0)
+            break;
+          puts(path);
+          ++printed;
+          break;
+        }
     }
   
   if (ferror (fp))
@@ -482,7 +473,7 @@ usage (stream, status)
 Usage: %s [-d path | --database=path] [-e | --existing]\n\
       [-i | --ignore-case] [--version] [--help] pattern...\n"),
 	   program_name);
-  fputs (_("\nReport bugs to <bug-findutils@gnu.org>."), stream);
+  fputs (_("\nReport bugs to <bug-findutils@gnu.org>.\n"), stream);
   exit (status);
 }
 
