@@ -27,12 +27,16 @@ Usage: updatedb [--localpaths='dir1 dir2...'] [--netpaths='dir1 dir2...']
 
   Report bugs to <bug-findutils@gnu.org>.
        "
-
+changeto=/
 old=no
 for arg
 do
-  opt=`echo $arg|sed 's/^\([^=]*\).*/\1/'`
-  val=`echo $arg|sed 's/^[^=]*=\(.*\)/\1/'`
+  # If we are unable to fork, the back-tick operator will 
+  # fail (and the shell will emit an error message).  When 
+  # this happens, we exit with error value 71 (EX_OSERR).
+  # Alternative candidate - 75, EX_TEMPFAIL.
+  opt=`echo $arg|sed 's/^\([^=]*\).*/\1/'`  || exit 71
+  val=`echo $arg|sed 's/^[^=]*=\(.*\)/\1/'` || exit 71
   case "$opt" in
     --localpaths) SEARCHPATHS="$val" ;;
     --netpaths) NETPATHS="$val" ;;
@@ -42,6 +46,7 @@ do
     --netuser) NETUSER="$val" ;;
     --localuser) LOCALUSER="$val" ;;
     --old-format) old=yes ;;
+    --changecwd)  changeto="$val" ;;
     --version) echo "GNU updatedb version @VERSION@"; exit 0 ;;
     --help) echo "$usage"; exit 0 ;;
     *) echo "updatedb: invalid option $opt
@@ -50,6 +55,13 @@ $usage" >&2
   esac
 done
 
+getuid() {
+    # format of "id" output is ...
+    # uid=1(daemon) gid=1(other)
+    # for `id's that don't understand -u
+    id | cut -d'(' -f 1 | cut -d'=' -f2
+}
+
 # You can set these in the environment, or use command-line options,
 # to override their defaults:
 
@@ -83,16 +95,24 @@ export TMPDIR
 : ${NETUSER=daemon}
 
 # The directory containing the subprograms.
-: ${LIBEXECDIR=@libexecdir@}
+if test -n "$LIBEXECDIR" ; then
+    : LIBEXECDIR already set, do nothing
+else
+    : ${LIBEXECDIR=@libexecdir@}
+fi
 
 # The directory containing find.
-: ${BINDIR=@bindir@}
+if test -n "$BINDIR" ; then
+    : BINDIR already set, do nothing
+else
+    : ${BINDIR=@bindir@}
+fi
 
 # The names of the utilities to run to build the database.
-: ${find=${BINDIR}/@find@}
-: ${frcode=${LIBEXECDIR}/@frcode@}
-: ${bigram=${LIBEXECDIR}/@bigram@}
-: ${code=${LIBEXECDIR}/@code@}
+: ${find:=${BINDIR}/@find@}
+: ${frcode:=${LIBEXECDIR}/@frcode@}
+: ${bigram:=${LIBEXECDIR}/@bigram@}
+: ${code:=${LIBEXECDIR}/@code@}
 
 PATH=/bin:/usr/bin:${BINDIR}; export PATH
 
@@ -114,14 +134,17 @@ trap 'rm -f $LOCATE_DB.n; exit' 1 15
 if test $old = no; then
 
 # FIXME figure out how to sort null-terminated strings, and use -print0.
-{
+if {
+cd "$changeto"
 if test -n "$SEARCHPATHS"; then
   if [ "$LOCALUSER" != "" ]; then
+    # : A1
     su $LOCALUSER -c \
     "$find $SEARCHPATHS \
      \\( $prunefs_exp \
      -type d -regex '$PRUNEREGEX' \\) -prune -o -print"
   else
+    # : A2
     $find $SEARCHPATHS \
      \( $prunefs_exp \
      -type d -regex "$PRUNEREGEX" \) -prune -o -print
@@ -129,14 +152,28 @@ if test -n "$SEARCHPATHS"; then
 fi
 
 if test -n "$NETPATHS"; then
-if [ "`id -u`" = 0 ]; then
+myuid=`getuid` 
+if [ "$myuid" = 0 ]; then
+    # : A3
     su $NETUSER -c \
-     "$find $NETPATHS \\( -type d -regex '$PRUNEREGEX' -prune \\) -o -print"
+     "$find $NETPATHS \\( -type d -regex '$PRUNEREGEX' -prune \\) -o -print" ||
+    exit $?
   else
-    $find $NETPATHS \( -type d -regex "$PRUNEREGEX" -prune \) -o -print
+    # : A4
+    $find $NETPATHS \( -type d -regex "$PRUNEREGEX" -prune \) -o -print ||
+    exit $?
   fi
 fi
 } | sort -f | $frcode > $LOCATE_DB.n
+then
+    # OK so far
+    true
+else
+    rv=$?
+    echo "Failed to generate $LOCATE_DB.n" >&2
+    rm -f $LOCATE_DB.n
+    exit $rv
+fi
 
 # To avoid breaking locate while this script is running, put the
 # results in a temp file, then rename it atomically.
@@ -168,25 +205,33 @@ trap 'rm -f $bigrams $filelist $LOCATE_DB.n; exit' 1 15
 # "to get everything in monotonic collating sequence, to avoid some
 # breakage i'll have to think about."
 {
+cd "$changeto"
 if test -n "$SEARCHPATHS"; then
   if [ "$LOCALUSER" != "" ]; then
+    # : A5
     su $LOCALUSER -c \
     "$find $SEARCHPATHS \
      \( $prunefs_exp \
-     -type d -regex '$PRUNEREGEX' \) -prune -o -print"
+     -type d -regex '$PRUNEREGEX' \) -prune -o -print" || exit $?
   else
+    # : A6
     $find $SEARCHPATHS \
      \( $prunefs_exp \
-     -type d -regex "$PRUNEREGEX" \) -prune -o -print
+     -type d -regex "$PRUNEREGEX" \) -prune -o -print || exit $?
   fi
 fi
 
 if test -n "$NETPATHS"; then
-  if [ "`id -u`" = 0 ]; then
+  myuid=`getuid`
+  if [ "$myuid" = 0 ]; then
+    # : A7
     su $NETUSER -c \
-     "$find $NETPATHS \\( -type d -regex '$PRUNEREGEX' -prune \\) -o -print"
+     "$find $NETPATHS \\( -type d -regex '$PRUNEREGEX' -prune \\) -o -print" ||
+    exit $?
   else
-    $find $NETPATHS \( -type d -regex "$PRUNEREGEX" -prune \) -o -print
+    # : A8
+    $find $NETPATHS \( -type d -regex "$PRUNEREGEX" -prune \) -o -print ||
+    exit $?
   fi
 fi
 } | tr / '\001' | sort -f | tr '\001' / > $filelist
